/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.testsys.lib;


import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Modul fuer Logging Informationen. Loginformationen knnen abgeschaltet und/oder in Datei
 * umgeleitet werden.
 * 
 * @author Chriesten
 *
 */

public class MyLogger
{
  /**
   * Instanz der Klasse; es gibt nur eine (Singleton)
   */
  public static MyLogger myLogger = null;

  /**
   * aktueller PrintStream
   */
  private PrintStream m_out = null;

  /**
   * liefert den aktuellen Ausgabestrom
   */
  public static PrintStream get_out()
  {
    
    return MyLogger.myLogger == null ? null : MyLogger.myLogger.m_out;
  }


  /**
   * Datum vor jede Ausgabezeile?
   */
  private Boolean m_mitDatum = false;
  
  /**
   * Ausgabe nach Kategorien filtern?
   */
  private String m_filterPat = null;

  
  /**
   * Ausgabe zusaetzlich auf System.out?
   */
  private Boolean m_auchSysOut = false;

  /**
   * nicht loggen?
   */
  private Boolean m_loggingAus = false;

  /**
   * vorherige Ausgabedatei
   */
  private static String _fileName_vorher = null;
  
  /**
   * aktuelle Ausgabedatei
   */
  private static String _fileName_akt = null;

  /**
   * voriger Initialisierungsmodus
   */
  private static long _modus_vorher;

  /**
   * aktueller Initialisierungsmodus
   */
  private static long _modus_akt;

  /**
   * Befindet sich die Ausgabe am Anfang einer neuen Zeile?
   */
  private Boolean m_neueZeile = true;
  
  /**
   * verdeckter Konstruktor. Wird von init() gerufen.
   */
  private MyLogger()
  {
  }

  /**
   * Initialisiert das Ausgabeziel.
   * Der Parameter dateiModus wird in dateiName, Modus und filterPat
   * gesplittet und {@link #init(String, long, String)}gerufen
   * @param dateiModus dateiName::modus::filterPat
   * @return true:ok, fehler sonst
   */
  public static synchronized Boolean init( String dateiModus)
  {
    String dateiName = null;
    String filterPat = null;
    long modus = 0;

    String[] modLogArgs = ArgumentParser.splitZk(dateiModus, "::", 3);
    try
    {
      if (modLogArgs.length >= 1)
        dateiName = modLogArgs[0];
      
      if (modLogArgs.length >= 2)
      {
        modus = new Integer(modLogArgs[1]);
      }
  
      
      if (modLogArgs.length >= 3)
      {
        filterPat = modLogArgs[2];
        if ( (filterPat==null) || filterPat.equalsIgnoreCase( "null") || filterPat.equalsIgnoreCase( ""))
          filterPat = null;
      }
    }  
    catch ( Exception e)
    {
      System.err.println("Aufruffehler: MyLogger("+dateiModus+").\n"
          + "korrekt: MyLogger(<datei>:<modus>:<filter>).");
      return(false);
    }

    Boolean bret = init( dateiName, modus, filterPat);
    return(bret);
  }

  /**
   * Initialisiert das Ausgabeziel.
   * 
   * @param dateiName Logging Informationen in diese Datei schreiben. <br>
   * Sonderziele: out, err, zurueck, null:  Ausgabe auf System.out, System.err, das letzte Ausgabeziel, nur Fehlerausgabe<br>
   * <UHRZEIT> im Dateinamen wird ersetzt durch Datum/Uhrzeit
   * @param modus 1: anhaengen<br>
   *              2: Datum vor jede Ausgabezeile<br>
   *              4: Zusaetzlich Ausgabe auf System.out<br>
   * @param filterPat nur Meldungen ausgeben, deren Kategorie filterPat entspricht
   * @return true: Initialisierung erfolgreich, false: Fehler
   */
  public static synchronized Boolean init( String dateiName, long modus, String filterPat)
  {
    //---------------------------------------------
    // vorhergehenden Output verwenden?
    //---------------------------------------------
    if ( (dateiName != null) && dateiName.equalsIgnoreCase( "null" ))
      dateiName = null;
    
    if ( (dateiName!=null) && (dateiName.equalsIgnoreCase("ZURUECK")) )
    {
      if (myLogger != null)
      {
        dateiName = _fileName_vorher;
        modus = _modus_vorher | 1;
      }
      else
        dateiName = null;
    }

    Boolean anhaengen = (modus & 1) == 1; 
    Boolean mitDatum =  (modus & 2) == 2;
    Boolean auchSysOut = (modus & 4) == 4;
    
    String fileName = ersetzeZeitToken( dateiName );

    _fileName_vorher = _fileName_akt;
    _modus_vorher = _modus_akt;
    _fileName_akt = fileName;
    _modus_akt = modus;
    
    //---------------------------------------------
    // alten Logger freigeben
    //---------------------------------------------
    if ( myLogger != null )
    {
      myLogger.close();
      myLogger = null;
    }

    //---------------------------------------------
    // neuen Logger instanziieren
    //---------------------------------------------
    myLogger = new MyLogger();
    if ( myLogger == null )
      return(false);
    
    // default == kein Logging
    if (fileName == null )
    {
      myLogger.m_loggingAus = true;
      return(true);
    }

    Boolean bret = myLogger.open( fileName, anhaengen ); 
    if ( !bret )
    {
      myLogger = null;
      return(false);
    }
      
    myLogger.m_mitDatum = mitDatum;
    myLogger.m_auchSysOut = auchSysOut;
    myLogger.m_filterPat = filterPat;

    return (bret);
  }

  public static synchronized void reinit()
  {
    if ( myLogger != null )
    {
      _fileName_vorher = _fileName_akt;
      myLogger.close();
      myLogger = null;
    }
  }

  /**
   * Schreibe Logging Info
   * 
   * @param ausgabe Ausgabe mit Zeilenumbruch.
   */
  public static void logln( String kategorie, String ausgabe )
  {
    MyLogger.privlog1( kategorie, ausgabe, true );
  }

  /**
   * Schreibe Logging Info
   * 
   * @param ausgabe Ausgabe, ohne Zeilenumbruch.
   */
  public static void log( String kategorie, String ausgabe )
  {
    MyLogger.privlog1( kategorie, ausgabe, false );
  }

  /**
   * Schreibe Logging Info; private Wrapper Funktion.
   * 
   * @param ausgabe Ausgabe
   * @param nl true: mit NewLine
   */
  private static void privlog1( String kategorie, String ausgabe, boolean nl )
  {
    // Logging abgeschaltet?
    if ( (MyLogger.myLogger != null) && myLogger.m_loggingAus )
    {
      // Fehler trotzdem loggen!
      if ( (MyLogger.myLogger != null) && (kategorie.equalsIgnoreCase( "ERROR") ) )
        MyLogger.myLogger.privlog( System.err, kategorie, ausgabe, nl );
      return;
    }
    
    if ( MyLogger.myLogger == null )
    {
      MyLogger.init(null, 1L, null);
      if ( MyLogger.myLogger == null )
      {
        return;
      }
    }

    MyLogger.myLogger.privlog( myLogger.m_out, kategorie, ausgabe, nl );
    if ( MyLogger.myLogger.m_auchSysOut && (MyLogger.myLogger.m_out != System.out) )
      MyLogger.myLogger.privlog( System.out, kategorie, ausgabe, nl );
  }

  /**
   * Schreibe Logging Info; private Aktionsfunktion.
   * 
   * @param kategorie Ausgabekategorie
   * @param ausgabe Ausgabe
   * @param nl true: mit NewLine
   */
  private void privlog( PrintStream ps, String kategorie, String ausgabe, boolean nl )
  {
    if ( (m_filterPat != null) && (!kategorie.equalsIgnoreCase("ERROR")) && (!kategorie.matches(m_filterPat)) )
    {
      return;
    }
    
    // Datum einfuegen?
    String meinDatum = "";
    if ( m_mitDatum && m_neueZeile )
    {
      SimpleDateFormat sdf = new SimpleDateFormat("yyMMdd_HHmmss_S ");
      meinDatum = sdf.format(new Date());
    }
    
    // Verarbeite kategorie
    String myKat = (kategorie == "" || !m_neueZeile) ? "" : kategorie+": ";

    if ( ps == null )
      return;

    try
    {
      ps.print( meinDatum + myKat + ausgabe );
      if ( nl )
      {
        ps.println( "" );
        m_neueZeile = true;
      }
      else
        m_neueZeile = false;

      ps.flush();
    }
    catch ( Exception e )
    {
      e.printStackTrace( System.err );
    }
  }

  /**
   * Oeffne Datei
   * 
   * @param fileName zu oeffnende Datei
   * @param append anhaengen?
   * @return true: open erfolgreich, false;: Fehler
   */
  private Boolean open( String fileName, boolean append )
  {
    if ( (fileName.equalsIgnoreCase( "zurueck" )) )
    {
      return (true);
    }

    if ( (fileName.equalsIgnoreCase( "out" )) || (fileName.equalsIgnoreCase( "System.out" )) )
    {
      m_out = System.out;
      return (true);
    }

    if ( (fileName.equalsIgnoreCase( "err" )) || (fileName.equalsIgnoreCase( "System.err" )) )
    {
      m_out = System.err;
      return (true);
    }

    try
    {
      m_out = new PrintStream( new FileOutputStream( fileName, append ) );
    }
    catch ( IOException e )
    {
      e.printStackTrace();
      return (false);
    }
    return (true);
  }

  /**
   * Schliesse Datei.
   */
  private void close()
  {
    if ( m_out != null )
    {
      m_out.flush();
    }

    if ( m_out != null && m_out != System.out && m_out != System.err )
    {
      try
      {
        m_out.close();
      }
      catch ( Exception e )
      {
        return;
      }
    }
    return;
  }
  
  /**
   * Ersetzt das token <UHRZEIT> in quelle durch Datum/Uhrzeit.
   * @param quelle
   * @return quelle mit der Ersetzung
   */
  private static String ersetzeZeitToken( String quelle )
  {
    if ( (quelle==null) || (quelle.indexOf( "<UHRZEIT>" ) == -1) )
      return(quelle);
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd_HHmmss");
    String uhrzeit = sdf.format(new Date());
    return quelle.replaceAll( "<UHRZEIT>", uhrzeit);
  }
  

  /**
   * Testroutine fuer das lokale Modul. Wird fuer verschiedene Tests im Laufe des Entwicklungs- und Wartungsprozesses verwendet
   * 
   * @param args
   *          Testargumente
   */
  public static void main( String[] args )
  {
    Boolean bret;

    bret = MyLogger.init( "out", 2, null);
    MyLogger.log( "X", "123 " );
    MyLogger.logln( "X", "456" );
    bret = MyLogger.init( "null", 2, null);
    MyLogger.logln( "ERROR", "789" );
    MyLogger.log( "X", "abc" );
    MyLogger.log( "X", "def" );
    bret = MyLogger.init( "out", 2, null);
    MyLogger.logln( "X", "ABC" );
    MyLogger.log( "X", "DEF" );
    MyLogger.logln( "", "1"+2+"33" );
    bret = MyLogger.init( "out", 2, "(BR)|(TTT_.*)");
    System.out.println(bret);
    MyLogger.logln( "BR", "A" );
    MyLogger.logln( "TTT_LH", "B" );
    MyLogger.logln( "BRXYZ", "C" );
    MyLogger.logln( "ABR", "D" );

  }
}