/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.testsys.testsys;


import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.testsys.lib.ArgumentParser;
import de.bsvrz.sys.testsys.lib.MyLogger;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hauptprogramm der SWE berprfung System ; Segment 10, SWE 10.2.
 *  
 * Startet die Anwendung "berprfung System". berprfung System initialisiert zuerst die DaV Verbindung und startet dann die Plugins.
 * Die Plugins fhren die jeweiligen berprfungen aus. 
 * 
 * @author Chriesten
 * 
 */
public class UeberpruefungSystemApp implements StandardApplication
{
  /**
   * Aktuelle Verbindung zum Datenverteiler. Wird fr JUNIT Test
   * gebraucht.
   */
  public static ClientDavInterface _statConnection;

  /**
   * Aktuelle Verbindung zum Datenverteiler.
   */
  private ClientDavInterface m_davConnection;

  /**
   * Die bergebenen plugins. Komma separierte Liste.
   */
  private String m_plugins = null;

  /**
   * Modulspezifische Logdatei
   */
  String m_modLog = null;
  
  /**
   * Instanz des PluginVerwalters
   */
  private PluginVerwalter m_pluginVerwalter;

  // -------------------------------------------------------------
  // StandardApplication, Schnittstellenfunktionen
  // -------------------------------------------------------------

  /*
   * (non-Javadoc)
   * 
   * @see sys.funclib.application.StandardApplication#initialize(stauma.dav.clientside.ClientDavInterface)
   */
  public void initialize( ClientDavInterface davConnection ) throws Exception
  {
    // DAV Objekte initialisieren
    m_davConnection = davConnection;
    _statConnection = m_davConnection;
    MyLogger.logln( "VERLAUF", "DAV initialisiert, initialize gerufen" );
  }

  /*
   * (non-Javadoc)
   * 
   * @see sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments( ArgumentList argumentList ) throws Exception
  {
    m_plugins = argumentList.fetchArgument( "-ueberpruefer" ).asNonEmptyString();

    // optionale Argumente
    try
    {
      // modulspezifische Logging
      String dateiModus = argumentList.fetchArgument( "-modLog" ).asString();
      MyLogger.init( dateiModus );
    }
    catch ( IllegalArgumentException e )
    {
    }


    // wegwerf - Argumente
    try
    {
      String wegwerfen = argumentList.fetchArgument( "-datenverteiler2" ).asNonEmptyString();
    }
    catch ( IllegalArgumentException e )
    {
    }
    
    // restliche Argumente berlesen -> Vermeidung von Fehlern
    argumentList.fetchUnusedArguments();
  }

  // -------------------------------------------------------------

  /**
   * Instanziiert die Plugins
   * 
   * @param pluginListe
   *          Doppelpunkt separierte Liste mit plugins
   */
  private void initPlugins( String pluginListe )
  {
    String[] pluginNamen = ArgumentParser.splitZk( pluginListe, ":", 0 );
    m_pluginVerwalter = new PluginVerwalter( pluginNamen, this.m_davConnection );

    for (String plugin : pluginNamen)
    {
      MyLogger.logln( "TTT_PLUGPARS", "|" + plugin + "|" );
    }
  }
  
  public void startUesys()
  {
    
  }

  /**
   * startet das Modul berprfung System (SWE 10.2).
   * 
   * @param args Argumente. Neben den DaV - sind die folgenden Argumente vorgesehen:
   * -ueberpruefer, Liste der Plugins<br>
   * -modLog, Einstellung des optionales modulspezifischen Loggings<br><br>
   * 
   * -ueberpruefer=<Doppelpunkt(:) separierte Liste mit berprfer Plugins>, Pluginparameter
   * werden mit ";" getrennt angehngt, z.B. (ohne Zeilenumbruch) <br>
   * -ueberpruefer=de.bsvrz.sys.testsys.modulUhrzeit.ModulUhrzeit:de.bsvrz.sys.testsys.modulStartStoppUeb.ModulStartStoppUeb;rechner=lokal: 
   * de.bsvrz.sys.testsys.modulApplUeb.ModulApplUeb<br>
   * berprferargumente  
   * <br><br>
   * 
   * -modLog=dateiName::modus::filter z.B. -modLog="/temp/TestTestSys/sys.log:1" <br>
   * modus 0: Datei berschreiben, 1: anhngen, 2: Datum vor jede Ausgabezeile, 4: Ausgabe zustzlich auf System.out <br>
   * dateiName: Ausgabedatei, null: Ausgaben unterdrcken<br>
   * filter, regulrer Ausdruck, mgliche EinzelausdrckeBM, VERLAUF, ERROR, TTT <br>
   * 
   * Beispielparameter:
   * -datenverteiler=192.0.1.97:8085
   * -debugLevelStdErrText=FINE
   * -benutzer=Tester
   * -authentifizierung=C:/Datenverteiler/skripte-dosshell/passwd
   * -inkarnationsName=Joachim
   * -modLog=C:/temp/TestTestSys/testSyslog.log::4::(BM|VERLAUF)
   * -ueberpruefer="de.bsvrz.sys.testsys.modulUhrzeit.ModulUhrzeit,de.bsvrz.sys.testsys.modulStartStoppUeb.ModulStartStoppUeb,de.bsvrz.sys.testsys.modulApplUeb.ModulApplUeb"
   */
  public static void main( String[] args )
  {
    // Beipielargumente:
    String[] myargs = new String[7];
    myargs[0] = "-datenverteiler=192.0.1.127:8083"; // globaler DAV
    myargs[1] = "-debugLevelStdErrText=INFO";
    myargs[2] = "-benutzer=Tester";
    myargs[3] = "-authentifizierung=C:/Datenverteiler/skripte-dosshell/passwd";
    myargs[4] = "-inkarnationsName=Joachim";
    myargs[5] = "-modLog=C:/temp/TestTestSys/log<UHRZEIT>.log::6::BM|ERROR|VERLAUF|TTT";
    // plugins
    String plugUhrzeit = "de.bsvrz.sys.testsys.modulUhrzeit.ModulUhrzeit";
    String plugStartStopp = "de.bsvrz.sys.testsys.modulStartStoppUeb.ModulStartStoppUeb;rechner=192.0.1.127";
    String plugApplUeb = "de.bsvrz.sys.testsys.modulApplUeb.ModulApplUeb";

    myargs[6] = "-ueberpruefer=" + plugUhrzeit + ":" + plugApplUeb + ":" + plugStartStopp;

//myargs[6] = "-ueberpruefer=de.bsvrz.sys.testsys.modulApplUeb.ModulApplUeb";
//args=myargs;
    
    System.out.println("-----------------------------------------");
    for (String arg : args)
    {
      System.out.println(arg);
    }
    System.out.println("-----------------------------------------");

    // OK Code
    UeberpruefungSystemApp uesys = new UeberpruefungSystemApp();
    StandardApplicationRunner.run( uesys, args );
    // m_plugins wird in der vom StandardApplicationRunner() gerufenen Routine parseArguments() gesetzt.
    uesys.initPlugins( uesys.m_plugins );
  }
}
